"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleKeydown = void 0;
const slate_1 = require("slate");
const list_utils_1 = require("./list-utils");
function handleKeydown(ev, editor) {
    const { selection } = editor;
    if (!selection || !slate_1.Range.isCollapsed(selection)) {
        return false;
    }
    const [start] = slate_1.Range.edges(selection);
    if (ev.key === "Tab") {
        if (moveCursor(editor, start)) {
            ev.preventDefault();
            return true;
        }
        if ((0, list_utils_1.moveListItem)(editor, start, ev.shiftKey)) {
            ev.preventDefault();
            return true;
        }
    }
    if (ev.key === "Enter") {
        if ((0, list_utils_1.addListSymbolToNewLine)(editor, start)) {
            ev.preventDefault();
            return true;
        }
    }
    return false;
}
exports.handleKeydown = handleKeydown;
function moveCursor(editor, location) {
    const after = slate_1.Editor.after(editor, location, {
        unit: "line",
    });
    const range = after && slate_1.Editor.range(editor, after, location);
    const afterText = range && slate_1.Editor.string(editor, range);
    if (!afterText) {
        return false;
    }
    return ["**", "*", "_", "`", "~"].some((i) => {
        if (afterText.startsWith(i)) {
            slate_1.Transforms.move(editor, { unit: "character", distance: i.length });
            if (afterText.endsWith(i)) {
                slate_1.Transforms.insertText(editor, " ");
            }
            return true;
        }
    });
}
